/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.faults;

import org.apache.ws.Soap1_1Constants;
import org.apache.ws.resource.i18n.Keys;
import org.apache.ws.resource.i18n.MessagesImpl;
import org.apache.ws.util.i18n.Messages;

import javax.xml.namespace.QName;

/**
 * A fault that is thrown when the expected resource identifier header element
 * is not found in a SOAP request Header. The name of the header element that
 * is expected for a particular type of resource is configured via the
 * resourceKeyName parameter for that resource's home in the Apache WSRF
 * JNDI configuration file.
 */
public class ResourceKeyHeaderNotFoundFaultException
        extends FaultException
{

    private static final Messages MSG = MessagesImpl.getInstance();

    /**
     * Creates a new {@link ResourceKeyHeaderNotFoundFaultException} object.
     *
     * @param expectedHeaderName DOCUMENT_ME
     */
    public ResourceKeyHeaderNotFoundFaultException( QName expectedHeaderName )
    {
        super( Soap1_1Constants.FAULT_CLIENT, MSG.getMessage( Keys.EXPECTED_ADDRESSING_HEADER, String.valueOf( expectedHeaderName ) ) );
    }

}