/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.handler;


import org.apache.ws.resource.metadataexchange.v2004_09.MetadataExchangeConstants;

import javax.xml.namespace.QName;
import java.util.HashMap;
import java.util.Map;

/**
 * Maps message-part element QNames from spec WSDLs to Java method names.
 *
 * @author Sal Campana, Ian P. Springer
 */
public class DefaultMethodMap
   extends AbstractSoapMethodNameMap
{

   private static final Map METHOD_MAP = initMap();

   private static Map initMap()
   {
      Map map = new HashMap(  );
      addWsrpOperations( map );
      addWsrlOperations( map );
      addWsxOperations( map );
      return map;
   }

   private static void addWsrpOperations( Map map )
   {
      //draft 01
      map.put( new QName( org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants.NSURI_WSRP_SCHEMA, "GetResourceProperty", org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants.NSPREFIX_WSRP_SCHEMA ),
                                         "getResourceProperty" );
      map.put( new QName( org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants.NSURI_WSRP_SCHEMA, "QueryResourceProperties", org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants.NSPREFIX_WSRP_SCHEMA ),
                                         "queryResourceProperties" );
      map.put( new QName( org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants.NSURI_WSRP_SCHEMA, "SetResourceProperties", org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants.NSPREFIX_WSRP_SCHEMA ),
                                         "setResourceProperties" );
      map.put( new QName( org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants.NSURI_WSRP_SCHEMA, "GetMultipleResourceProperties", org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants.NSPREFIX_WSRP_SCHEMA ),
                                         "getMultipleResourceProperties" );

      //draft 05
      map.put( new QName( org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants.NSURI_WSRP_SCHEMA, "GetResourceProperty", org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants.NSPREFIX_WSRP_SCHEMA ),
                                         "getResourceProperty" );
      map.put( new QName( org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants.NSURI_WSRP_SCHEMA, "QueryResourceProperties", org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants.NSPREFIX_WSRP_SCHEMA ),
                                         "queryResourceProperties" );
      map.put( new QName( org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants.NSURI_WSRP_SCHEMA, "SetResourceProperties", org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants.NSPREFIX_WSRP_SCHEMA ),
                                         "setResourceProperties" );
      map.put( new QName( org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants.NSURI_WSRP_SCHEMA, "GetMultipleResourceProperties", org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants.NSPREFIX_WSRP_SCHEMA ),
                                         "getMultipleResourceProperties" );

      map.put( new QName( org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants.NSURI_WSRP_SCHEMA, "GetResourcePropertyDocument", org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants.NSPREFIX_WSRP_SCHEMA ),
                                               "getResourcePropertyDocument" );
      map.put( new QName( org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants.NSURI_WSRP_SCHEMA, "InsertResourceProperties", org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants.NSPREFIX_WSRP_SCHEMA ),
                                                "insertResourceProperties" );
      map.put( new QName( org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants.NSURI_WSRP_SCHEMA, "UpdateResourceProperties", org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants.NSPREFIX_WSRP_SCHEMA ),
                                               "updateResourceProperties" );
      map.put( new QName( org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants.NSURI_WSRP_SCHEMA, "DeleteResourceProperties", org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants.NSPREFIX_WSRP_SCHEMA ),
                                                "deleteResourceProperties" );

   }
   
   private static void addWsrlOperations( Map map )
   {
      //draft 01
      map.put( new QName( org.apache.ws.resource.lifetime.v2004_06.ResourceLifetimeConstants.NSURI_WSRL_SCHEMA, "SetTerminationTime", org.apache.ws.resource.lifetime.v2004_06.ResourceLifetimeConstants.NSPREFIX_WSRL_SCHEMA ),
                                         "setTerminationTime" );
      map.put( new QName( org.apache.ws.resource.lifetime.v2004_06.ResourceLifetimeConstants.NSURI_WSRL_SCHEMA, "Destroy", org.apache.ws.resource.lifetime.v2004_06.ResourceLifetimeConstants.NSPREFIX_WSRL_SCHEMA ),
                                         "destroy" );

      //draft 04
      map.put( new QName( org.apache.ws.resource.lifetime.v2004_11.ResourceLifetimeConstants.NSURI_WSRL_SCHEMA, "SetTerminationTime", org.apache.ws.resource.lifetime.v2004_11.ResourceLifetimeConstants.NSPREFIX_WSRL_SCHEMA ),
                                         "setTerminationTime" );
      map.put( new QName( org.apache.ws.resource.lifetime.v2004_11.ResourceLifetimeConstants.NSURI_WSRL_SCHEMA, "Destroy", org.apache.ws.resource.lifetime.v2004_11.ResourceLifetimeConstants.NSPREFIX_WSRL_SCHEMA ),
                                         "destroy" );
   }

    /**
     * Adds the WS-MetadataExchange operation and Action String to the map
     * @param map
     */
   private static void addWsxOperations(Map map )
   {
       //put this in as string for looking up the "Get/Request" operation
       map.put(MetadataExchangeConstants.ACTION_GET,"get");
       //put this in as string for looking up the "GetMetadata/Request" operation
       map.put(MetadataExchangeConstants.ACTION_GETMETADATA, "getMetadata");
   }

   private SoapMethodNameMap m_parent;

   /**
    * Creates a new {@link DefaultMethodMap} object.
    *
    * @param parent DOCUMENT_ME
    */
   public DefaultMethodMap( SoapMethodNameMap parent )
   {
      m_parent = parent;
   }

   /**
    * DOCUMENT_ME
    *
    * @param request DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getDefaultMethodName( QName request )
   {
      return (String) METHOD_MAP.get( request );
   }

    /**
     * Returns a Default name for a method based on a request's WS-Addressing Action Header.
     *
     * @param action String representation of WS-Addressing Action Header Value
     * @return The name of the operation OR null if there is no Action registered.
     */
    public String getSpecActionMethodName(String action)
    {
        return (String) METHOD_MAP.get( action );
    }

    /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public SoapMethodNameMap getParent(  )
   {
      return m_parent;
   }
}