package org.apache.ws.resource.lifetime.impl;

import org.apache.ws.resource.lifetime.ResourceTerminationEvent;
import org.apache.ws.resource.properties.NamespaceVersionHolder;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlString;

import java.util.Calendar;


/**
 * @author Sal Campana
 */
public class ResourceTerminationEventImpl extends AbstractResourceTerminationEvent
        implements ResourceTerminationEvent
{
    /**
     * Sets termination time and reason to passed in params.
     *
     * @param resourceId      The resource Id of the resource being destroyed.
     * @param terminationTime
     * @param reason
     */
    public ResourceTerminationEventImpl(Object resourceId, Calendar terminationTime, Object reason)
    {
        super(resourceId, terminationTime, reason);
    }

    /**
     * Constructor creates termination time upon construction.  Reason is null.
     *
     * @param resourceId The resource Id of the resource being destroyed.
     */
    public ResourceTerminationEventImpl(Object resourceId)
    {
        super(resourceId);
    }

    /**
     * Set the reson to the object specified and termination time is set upon construction.
     *
     * @param resourceId The resource Id of the resource being destroyed.
     * @param reason
     */
    public ResourceTerminationEventImpl(Object resourceId, Object reason)
    {
        super(resourceId, reason);
    }


    public XmlObject getTerminationNotifDocXmlBean(NamespaceVersionHolder namespaces)
    {
        Object reason = getReason();
        if (!(reason instanceof XmlObject))
        {
            reason = XmlString.Factory.newInstance();
            ((XmlString) reason).setStringValue(reason.toString());
        }

        if (namespaces instanceof org.apache.ws.resource.properties.v2004_11.impl.NamespaceVersionHolderImpl)
        {
            org.oasisOpen.docs.wsrf.x2004.x11.wsrfWSResourceLifetime12Draft04.TerminationNotificationDocument notifDoc = org.oasisOpen.docs.wsrf.x2004.x11.wsrfWSResourceLifetime12Draft04.TerminationNotificationDocument.Factory.newInstance();
            org.oasisOpen.docs.wsrf.x2004.x11.wsrfWSResourceLifetime12Draft04.TerminationNotificationDocument.TerminationNotification terminationNotification = notifDoc.addNewTerminationNotification();
            terminationNotification.setTerminationTime(getTerminationTime());

            if (reason != null)
            {
                terminationNotification.setTerminationReason((XmlObject) reason);
            }

            return notifDoc;
        }
        else if (namespaces instanceof org.apache.ws.resource.properties.v2004_06.impl.NamespaceVersionHolderImpl)
        {
            org.oasisOpen.docs.wsrf.x2004.x06.wsrfWSResourceLifetime12Draft01.TerminationNotificationDocument notifDoc = org.oasisOpen.docs.wsrf.x2004.x06.wsrfWSResourceLifetime12Draft01.TerminationNotificationDocument.Factory.newInstance();
            org.oasisOpen.docs.wsrf.x2004.x06.wsrfWSResourceLifetime12Draft01.TerminationNotificationDocument.TerminationNotification terminationNotification = notifDoc.addNewTerminationNotification();
            terminationNotification.setTerminationTime(getTerminationTime());


            if (reason != null)
            {
                terminationNotification.setTerminationReason((XmlObject) reason);
            }

            return notifDoc;
        }
        else throw new org.apache.ws.resource.impl.UnsupportedVersionException(namespaces);


    }

}
