/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.properties.query.impl;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.XmlObjectWrapper;
import org.apache.ws.resource.i18n.Keys;
import org.apache.ws.resource.i18n.MessagesImpl;
import org.apache.ws.resource.properties.query.QueryExpression;
import org.apache.ws.util.XmlBeanUtils;
import org.apache.ws.util.i18n.Messages;
import org.apache.ws.util.xml.NamespaceContext;
import org.apache.ws.util.xml.impl.DomNamespaceContext;
import org.apache.xmlbeans.XmlObject;
import org.oasisOpen.docs.wsrf.x2004.x06.wsrfWSResourceProperties12Draft01.QueryExpressionType;

import java.net.URI;
import java.net.URISyntaxException;

/**
 * LOG-DONE
 *
 * @author Ian P. Springer
 */
public class XmlBeansQueryExpression
        implements QueryExpression, XmlObjectWrapper
{

    private static final Log LOG = LogFactory.getLog( XmlBeansQueryExpression.class );
    private static final Messages MSG = MessagesImpl.getInstance();

    private XmlObject m_queryExprXBean;
    private URI m_dialect;
    private Object m_content;
    private NamespaceContext m_nsContext;

    /**
     * Creates a new {@link XmlBeansQueryExpression} object.
     *
     * @param queryExprXBean DOCUMENT_ME
     */
    public XmlBeansQueryExpression( QueryExpressionType queryExprXBean )
    {
        this( queryExprXBean, getDialect( queryExprXBean ) );
    }

    /**
     * Creates a new {@link XmlBeansQueryExpression} object.
     *
     * @param queryExprXBean DOCUMENT_ME
     */
    public XmlBeansQueryExpression(org.oasisOpen.docs.wsrf.x2004.x11.wsrfWSResourceProperties12Draft05.QueryExpressionType queryExprXBean)
    {
        this( queryExprXBean, getDialect( queryExprXBean ) );
    }

    protected XmlBeansQueryExpression( XmlObject xBean, URI dialect )
    {
        m_queryExprXBean = xBean;
        m_dialect = dialect;
        XmlObject[] childElems = XmlBeanUtils.getChildElements( m_queryExprXBean );
        if ( childElems.length > 1 )
        {
            throw new IllegalArgumentException( MSG.getMessage( Keys.QUERY_ONLY_ONE_NODE ) );
        }
        if ( childElems.length == 1 )
        {
            m_content = childElems[0];
        }
        else
        {
            m_content = XmlBeanUtils.getValue( m_queryExprXBean );
        }
        m_nsContext = new DomNamespaceContext( m_queryExprXBean.newDomNode() );
        if ( LOG.isDebugEnabled() )
        {
            LOG.debug( MSG.getMessage( Keys.QUERY_EXPR, toString() ) );
        }
    }

    private static URI getDialect(
            org.oasisOpen.docs.wsrf.x2004.x11.wsrfWSResourceProperties12Draft05.QueryExpressionType queryExprElem )
    {
        try
        {
            return new URI( queryExprElem.getDialect() );
        }
        catch ( URISyntaxException urise )
        {
            throw new RuntimeException( MSG.getMessage( Keys.FAILED_INIT_DIALECT, urise ) );
        }
    }

    private static URI getDialect(
            org.oasisOpen.docs.wsrf.x2004.x06.wsrfWSResourceProperties12Draft01.QueryExpressionType queryExprElem )
    {
        try
        {
            return new URI( queryExprElem.getDialect() );
        }
        catch ( URISyntaxException urise )
        {
            throw new RuntimeException( MSG.getMessage( Keys.FAILED_INIT_DIALECT, urise ) );
        }
    }

    /**
     * DOCUMENT_ME
     *
     * @return DOCUMENT_ME
     */
    public Object getContent()
    {
        return m_content;
    }

    /**
     * DOCUMENT_ME
     *
     * @return DOCUMENT_ME
     */
    public URI getDialect()
    {
        return m_dialect;
    }

    public NamespaceContext getNamespaceContext()
    {
        return m_nsContext;
    }

    public void setNamespaceContext( NamespaceContext nsContext )
    {
        m_nsContext = nsContext;
    }

    /**
     * DOCUMENT_ME
     *
     * @return DOCUMENT_ME
     */
    public String toString()
    {
        return new ToStringBuilder( this ).append( MSG.getMessage( Keys.DIALECT ), m_dialect ).append(
                MSG.getMessage( Keys.CONTENT ), m_content )
                .toString();
    }

    public XmlObject getXmlObject()
    {
        return m_queryExprXBean;
    }

}