package org.apache.ws.resource.tool.velocity;

import org.apache.ws.resource.ResourceDefinition;
import org.apache.ws.util.XmlBeanNameUtils;

import javax.xml.namespace.QName;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;

/**
 * A bean that contains all properties that are needed by our Velocity templates.
 *
 * @author Sal Campana
 */
public class ServiceProperties
{
    String m_packageName;
    String m_serviceName;
    String m_serviceImplements;
    String m_namespace;
    String m_prefix;
    Map m_customMappings;
    Map m_propertyMap;
    String m_resourceImplements;
    String m_propertyDocumentName;
    String m_wsdlName;
    QName m_resourcekey;
    List m_resourceIncludes = new ArrayList();
    List m_serviceIncludes = new ArrayList();
    List m_resourceInitIncludes = new ArrayList();
    List m_abstractResourceInitIncludes = new ArrayList();
    private boolean m_hasProperties;
    private String m_endpointServiceName;
    private String m_endpointURL;
    private String m_jarDir;
    private String m_classesDir;
    private String m_srcDir;
    private String m_jarFileName;
    private ResourceDefinition m_resourceDef;
    private String m_namespaceVersionHolder;
    public static final String SPEC_NAMESPACE_MEMBER_NAME = "SPEC_NAMESPACE_SET";
    private QName m_serviceQName;
    private String m_portName;
    private QName m_portTypeQName;
    private List m_implementedProps = new ArrayList();
    private String m_webbAppName;
    private String m_namespaceVersionHolderClassName;

    public ServiceProperties(ResourceDefinition resourceDef)
    {
        m_resourceDef = resourceDef;
    }

    public String getSrcDir()
    {
        return m_srcDir;
    }

    public void setSrcDir(String srcDir)
    {
        m_srcDir = srcDir;
    }

    public String getClassesDir()
    {
        return m_classesDir;
    }

    public void setClassesDir(String classesDir)
    {
        m_classesDir = classesDir;
    }

    public String getJarDir()
    {
        return m_jarDir;
    }

    public void setJarDir(String jarDir)
    {
        m_jarDir = jarDir;
    }

    public List getResourceIncludeFiles()
    {
        return m_resourceIncludes;
    }

    public void addResourceIncludeFile( String resourceInclude )
    {
        m_resourceIncludes.add( resourceInclude );
    }

    public List getServiceIncludeFiles()
    {
        return m_serviceIncludes;
    }

    public void addServiceIncludeFile( String serviceInclude )
    {
        m_serviceIncludes.add( serviceInclude );
    }

    public String getPackageName()
    {
        return m_packageName;
    }

    public void setPackageName(String packageName)
    {
        this.m_packageName = packageName;
    }

    public String getServiceName()
    {
        return m_serviceName;
    }

    public void setServiceName(String serviceName)
    {
        this.m_serviceName = serviceName;
    }

    public String getServiceImplements()
    {
        return m_serviceImplements;
    }

    public void setServiceImplements(String serviceImplements)
    {
        this.m_serviceImplements = serviceImplements;
    }

    public String getNamespace()
    {
        return m_namespace;
    }

    public void setNamespace(String namespace)
    {
        this.m_namespace = namespace;
    }

    public String getPrefix()
    {
        return m_prefix;
    }

    public void setPrefix(String prefix)
    {
        this.m_prefix = prefix;
    }

    public Map getCustomMappings()
    {
        return m_customMappings;
    }

    public void setCustomMappings(Map customMappings)
    {
        this.m_customMappings = customMappings;
    }

    public Map getPropertyMap()
    {
        return m_propertyMap;
    }

    public void setPropertyMap(Map propertyMap)
    {
        this.m_propertyMap = propertyMap;
    }

    public String getResourceImplements()
    {
        return m_resourceImplements;
    }

    public void setResourceImplements(String resourceImplements)
    {
        this.m_resourceImplements = resourceImplements;
    }

    public String getPropertyDocumentName()
    {
        return m_propertyDocumentName;
    }

    public void setPropertyDocumentName(String propertyDocumentName)
    {
        this.m_propertyDocumentName = propertyDocumentName;
    }

    public String getWsdlName()
    {
        return m_wsdlName;
    }

    public void setWsdlName(String wsdlName)
    {
        this.m_wsdlName = wsdlName;
    }

    public QName getResourcekey()
    {
        return m_resourcekey;
    }

    public void setResourcekey(QName resourcekey)
    {
        this.m_resourcekey = resourcekey;
    }

    public void setHasProperties(boolean hasProps)
    {
        m_hasProperties = hasProps;
    }

    public boolean getHasProperties()
    {
        return m_hasProperties;
    }

    public void setEndpointServiceName(String serviceName)
    {
        m_endpointServiceName = serviceName;
    }

    public String getEndpointServiceName()
    {
        return m_endpointServiceName;
    }

    public void setEndpointURL(String enpointURL)
    {
       m_endpointURL = enpointURL;
    }

    public String getEndpointURL()
    {
        return m_endpointURL;
    }

    public String getJarFileName()
    {
        return m_jarFileName;
    }

    public void setJarFileName( String jarFileName )
    {
        m_jarFileName = jarFileName;
    }

    public boolean implementsResourceCapability(QName capability)
    {
        return m_resourceDef.implementsResourceCapability(capability);     
    }

    public List getResourceInitIncludeLocations()
    {
        return m_resourceInitIncludes;
    }

    public void addResourceInitIncludeLocation( String resourceInitInclude )
    {
        m_resourceInitIncludes.add( resourceInitInclude );
    }

    public List getAbstractResourceInitIncludeLocations()
    {
        return m_abstractResourceInitIncludes;
    }

    public void addAbstractResourceInitIncludeLocation( String abstractResourceInitInclude )
    {
        m_abstractResourceInitIncludes.add( abstractResourceInitInclude );
    }

    public String getNamespaceVersionHolder()
    {
        return m_namespaceVersionHolder;
    }

    public void setNamespaceVersionHolder(Class namespaceValueHolder)
    {
        if(namespaceValueHolder != null)
        {
            setNamespaceVersionHolderClassName(namespaceValueHolder);
            String className = namespaceValueHolder.getName();
        m_namespaceVersionHolder = "public static final "+className+" "+SPEC_NAMESPACE_MEMBER_NAME+" = new " + className + "();";
        }
    }
    public void setNamespaceVersionHolderClassName(Class namespaceValueHolder)
    {
        m_namespaceVersionHolderClassName = namespaceValueHolder.getName();
    }

    public String getNamespaceVersionHolderClassName()
    {
        return m_namespaceVersionHolderClassName;
    }

    public void setPortName(String portName)
    {
       m_portName = portName;
    }

    public void setPortTypeQName(QName portTypeQName)
    {
       m_portTypeQName = portTypeQName;
    }

    public void setServiceQName(QName serviceQname)
    {
         m_serviceQName = serviceQname;
    }

    public String getPortName()
    {
        return m_portName;
    }

    public QName getPortTypeQName()
    {
        return m_portTypeQName;
    }

    public QName getServiceQName()
    {
        return m_serviceQName;
    }

    public String getDocumentElementXmlBeanClassName(QName prop_qname)
    {
        return XmlBeanNameUtils.getDocumentElementXmlBeanClassName(prop_qname);
    }

    /**
     * Use this method to tell the ServiceProperties which ResourceProperties we implement for the user.
     *
     * @param propQnameTerminationTime
     */
    public void addImplementedProperty(QName propQnameTerminationTime)
    {
        m_implementedProps.add(propQnameTerminationTime);
    }

    /**
     * Returns a list of only the properties which the implementor has defined
     * in their wsdl AND that ARE implemented for them.
     *
     * @return A list of the props which the implementor will not need to implement.
     */
    public List getImplementedProperties()
    {
        List list = new ArrayList();
        for (int i = 0; i < m_implementedProps.size(); i++)
        {
            if (m_propertyMap.values().contains(m_implementedProps.get(i)))
            {
                list.add(m_implementedProps.get(i));
            }
        }
        return list;
    }

    /**
     * Returns a Map of the properties which the implementor will
     * need to implement.
     *
     * @return The Map of props which need to be implenmented/
     */
    public Map getImplementableProperties()
    {
        Map implementableMap = new HashMap(m_propertyMap);
        for (int i = 0; i < m_implementedProps.size(); i++)
        {
            implementableMap.remove( ((QName)m_implementedProps.get(i)).getLocalPart().toUpperCase());
        }
        return implementableMap;
    }

    public void setWebappName(String name)
    {
        m_webbAppName = name;
    }

    public String getWebappName()
    {
        return m_webbAppName;
    }
}
