/*=============================================================================*
 *  Copyright 2005 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.util.xml;

import java.util.Iterator;

/**
 * Interface for read only XML Namespace context processing.
 *
 * @author Ian Springer (ian DOT springer AT hp DOT com)
 */
public interface NamespaceContext
{

    /**
     * Get Namespace URI bound to a prefix in the current scope.
     *
     * @param prefix the prefix to resolve
     *
     * @return
     */
    String getNamespaceURI(String prefix);

    /**
     * Get prefix bound to Namespace URI in the current scope.
     *
     * @param namespaceURI
     * @return
     */
    String getPrefix(String namespaceURI);

    /**
     * Get all prefixes bound to a Namespace URI in the current scope.
     *
     * @param namespaceURI
     * @return
     */
    Iterator getPrefixes(String namespaceURI);

}
